@@ -83,8 +83,6 @@ module Network.CGI (
   , setCookie, deleteCookie
   -- * URL encoding
   , formEncode, urlEncode, formDecode, urlDecode
-  -- * Compatibility with the old API
-  , module Network.CGI.Compat
   ) where
 
 import Control.Exception (Exception(..))
@@ -107,7 +105,6 @@ import Network.CGI.RFC822Headers (ContentType(..),
                                   parseContentType, showContentType)
 import Network.CGI.Monad
 import Network.CGI.Protocol
-import Network.CGI.Compat
 
 import Text.XHtml (Html, renderHtml, header, (<<), thetitle, (+++), 
                    body, h1, paragraph, hr, address)
